<?php
namespace FiloBlu\Checkproducts\Cron;
use \Psr\Log\LoggerInterface;

class Pricecheck {
    protected $_logger;
    protected $_productRepository;
    protected $_searchCriteriaBuilder;
    protected $_scopeConfig;
    protected $_storeRepository;
    protected $_objectManager;
    protected $_linkManagement;

    public function __construct(
        LoggerInterface $logger,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreRepository $storeRepository,
        \Magento\Framework\ObjectManagerInterface $objectmanager,
        \Magento\ConfigurableProduct\Api\LinkManagementInterface $linkManagement
    ) {
        $this->_logger = $logger;
        $this->_productRepository = $productRepository;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeRepository = $storeRepository;
        $this->_objectManager = $objectmanager;
        $this->_linkManagement = $linkManagement;
    }

    /**
     * Write to system.log
     *
     * @return void
     */

    public function execute() {

        $active = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_general/checkproducts_enabled');
        if(!$active){
            return;
        }

        $searchCriteria = $this->_searchCriteriaBuilder->addFilter('status', '1', 'eq')->create();
        $searchResults = $this->_productRepository->getList($searchCriteria);
        $products = $searchResults->getItems();
        $stores = $this->_storeRepository->getList();
        $problems = [];
        foreach ($products as $product){
            if(isset($product['manual_status']) && intval($product['manual_status'])){
                continue;
            }
            $check = [];
            $type = $product['type_id'];
            foreach ($stores as $store) {
                $product = $this->_productRepository->get($product['sku'], true, $store["store_id"], true);

                $priceCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_price', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                if($priceCheck && $type != 'configurable'){
                    if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
                        $check['price'][$store["code"]] = false;
                    }
                }
                /*elseif($priceCheck && $type == 'configurable' && intval($store["store_id"]) == 0){
                    $res = $this->checkConfigurablePrice($product);
                    if(!$res){
                        $check['price_configurable'][$store["code"]] = false;
                    }
                }*/

                if(intval($store["store_id"]) == 0) {
                    $categoryCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    if ($categoryCheck) {
                        if (!isset($product['category_ids']) || (isset($product['category_ids']) && count($product['category_ids']) <= 0)) {
                            $check['category'][$store["code"]] = false;
                        }
                    }

                    $imageCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_image', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    if ($imageCheck) {
                        if(!$product->getImage() || trim($product->getImage()) == 'no_selection'){
                            $check['images'][$store["code"]] = false;
                        }
                        /*if (!isset($product['media_gallery']) || (isset($product['media_gallery']) && count($product['media_gallery']) <= 0)) {
                            if (!isset($product['media_gallery']['images']) || (isset($product['media_gallery']['images']) && count($product['media_gallery']['images']) <= 0)) {
                                $check['images'][$store["code"]] = false;
                            }
                        }*/
                    }
                }
            }
            if(count($check) > 0) {
                $attributesData['status'] = 0;
                $attributesData['to_check'] = 1;
                if(!isset($product['name']) || (isset($product['name']) && trim($product['name']) == '')){
                    $attributesData['name'] = 'NO NAME';
                }
                try {
                    $this->_objectManager->get('Magento\Catalog\Model\Product\Action')->updateAttributes([$product['entity_id']], $attributesData, 0);
                    //$this->_productRepository->save($product);
                }
                catch (\Exception $e){
                    $this->_logger->info('Product: '.$product['sku'] . " Error: " . $e->getMessage());
                }
                $problems[$product['sku']] = $check;
            }
        }

        $this->sendReport($problems);
        $this->_logger->info('End Price Check');
    }

    public function sendReport($problems){
        /* Here we prepare data for our email  */

        $sender = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_sender', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderEmail = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderName = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $receiversEmail = explode(',', $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_receiver', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));


        /* Receiver Detail  */
//        $receiverInfo = [
//            'name' => '',
//            'email' => 'luigi+test@filoblu.com'
//        ];


        /* Sender Detail  */
        $senderInfo = [
            'name' => $senderName,
            'email' => $senderEmail,
        ];


        /* Assign values for your template variables  */
        $emailTemplateVariables = [];
        $emailTempVariables['cron'] = 'Price Check';

        $tableBodyPrices = '';
        $tableBodyCategory = '';
        foreach($problems as $key => $values){

            foreach ($values as $value => $stores ){
                if($value == 'price') {
                    $tableBodyPrices .= '<tr>';
                    $tableBodyPrices .= '<td>'. $key .'</td>';
                    $tableBodyPrices .= '<td>Price less than 0</td>';
                    $tableBodyPrices .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyPrices .= '</tr>';
                }
                elseif ($value == 'price_configurable'){
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>'. $key .'</td>';
                    $tableBodyCategory .= '<td>All children have zero price</td>';
                    $tableBodyCategory .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyCategory .= '</tr>';
                }
                elseif ($value == 'category'){
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>'. $key .'</td>';
                    $tableBodyCategory .= '<td>Without category</td>';
                    $tableBodyCategory .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyCategory .= '</tr>';
                }
                elseif ($value == 'images'){
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>'. $key .'</td>';
                    $tableBodyCategory .= '<td>Image not found</td>';
                    $tableBodyCategory .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                    $tableBodyCategory .= '</tr>';
                }
            }
        }


        $emailTempVariables['tBodyPrices'] = $tableBodyPrices;

        $emailTempVariables['tBodyCategory'] = $tableBodyCategory;

        /* call send mail method from helper or where you define it*/
        $this->_objectManager->get('FiloBlu\Checkproducts\Helper\Email')->sendCustomEmail(
            $emailTempVariables,
            $senderInfo,
            $receiversEmail,
            'filoblu_checkproducts/checkproducts_email/checkproducts_email_template_price'
        );
    }

    public function checkConfigurablePrice($configurableProduct){
        $childProducts = $this->_linkManagement->getChildren($configurableProduct['sku']);
        foreach ($childProducts as $children){
            $childrenPrice = true;
            $stores = $this->_storeRepository->getList();
            foreach ($stores as $store) {
                $product = $this->_productRepository->get($children['sku'], true, $store["store_id"], true);
                if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
                    $childrenPrice = false;
                    Break;
                }
            }
            if($childrenPrice == true){
                return true;
            }
        }
        return false;
    }

}